/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla networking code.
 *
 * The Initial Developer of the Original Code is
 * Google Inc.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Christian Biesinger <cbiesinger@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"
%{C++
namespace mozilla {
class TimeStamp;
}
%}

native TimeStamp(mozilla::TimeStamp);

// All properties return zero if the value is not available
[scriptable, uuid(c259b593-a9bf-4d08-8149-ef89e1977dc4)]
interface nsITimedChannel : nsISupports {
  // Set this attribute to true to enable collection of timing data.
  // channelCreationTime will be available even with this attribute set to
  // false.
  attribute boolean timingEnabled;

  [notxpcom] readonly attribute TimeStamp channelCreation;
  [notxpcom] readonly attribute TimeStamp asyncOpen;

  // The following are only set when the document is not (only) read from the
  // cache
  [notxpcom] readonly attribute TimeStamp domainLookupStart;
  [notxpcom] readonly attribute TimeStamp domainLookupEnd;
  [notxpcom] readonly attribute TimeStamp connectStart;
  [notxpcom] readonly attribute TimeStamp connectEnd;
  [notxpcom] readonly attribute TimeStamp requestStart;
  [notxpcom] readonly attribute TimeStamp responseStart;
  [notxpcom] readonly attribute TimeStamp responseEnd;

  // The following are only set if the document is (partially) read from the
  // cache
  [notxpcom] readonly attribute TimeStamp cacheReadStart;
  [notxpcom] readonly attribute TimeStamp cacheReadEnd;

  // All following are PRTime versions of the above.
  readonly attribute PRTime channelCreationTime;
  readonly attribute PRTime asyncOpenTime;
  readonly attribute PRTime domainLookupStartTime;
  readonly attribute PRTime domainLookupEndTime;
  readonly attribute PRTime connectStartTime;
  readonly attribute PRTime connectEndTime;
  readonly attribute PRTime requestStartTime;
  readonly attribute PRTime responseStartTime;
  readonly attribute PRTime responseEndTime;
  readonly attribute PRTime cacheReadStartTime;
  readonly attribute PRTime cacheReadEndTime;
};
